package edu.uoc.tfc.glucodiari;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.res.Configuration;
import android.graphics.Color;
import android.graphics.PorterDuff;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;

import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Locale;

/**
 * Aquesta classe permet configurar el calendari segons el mes i l'any seleccionat.
 * També infla el menú per accedir a les altres opcions de l'aplicació.
 * @author: Xavier Asensio - (TFC App. Android - UOC).
 * @version: 1.0
 */

public class Calendari extends Activity {

    private int mes;
    private int any;
    private int primerDia;
    private boolean mostrarColor;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        setContentView(R.layout.activity_calendari);

        /* Aplicar l'idioma seleccionat en les preferències */
        AplicaIdioma();

        /* Recupera si es vol colorejar el calendari i configura el switch*/
        mostrarColor = new Opcions().getColorejar(this);
        Switch switchColorejar = (Switch) findViewById(R.id.switchMostrarColor);
        switchColorejar.setChecked(mostrarColor);
        switchColorejar.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if(isChecked){
                    mostrarColor = true;
                    SwitchActualitzat();
                }else{
                    mostrarColor = false;
                    SwitchActualitzat();
                }
            }
        });

        /* Recuperar l'any i el mes actuals */
        RecuperaMesAny();

        /* Configurar el calendari */
        ActualitzaCalendari();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_calendari, menu);

        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        // Handle item selection
        Intent intent;
        switch (item.getItemId()) {
            case R.id.menuDades:
                /* Llança l'activitat Eines de Dades */
                intent = new Intent(Calendari.this, EinesDeDades.class);
                startActivity(intent);
                return true;
            case R.id.menuInformes:
                 /* Llança l'activitat Informes */
                intent = new Intent(Calendari.this, Informes.class);
                startActivity(intent);
                return true;
            case R.id.menuInfo:
                /* Llança l'activitat Informació */
                intent = new Intent(Calendari.this, Info.class);
                startActivity(intent);
                return true;
            case R.id.menuConfiguracio:
                /* Llança l'activitat Configuració */
                intent = new Intent(Calendari.this, Configuracio.class);
                startActivityForResult(intent, 0);
                return true;
            case R.id.menuSortir:
                sortir();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    /**
     * Metode que guarda el nou valor del switch quan s'ha modificat.
     */
    private void SwitchActualitzat(){
        new Opcions().setColorejar(this, mostrarColor);
        ActualitzaCalendari();
    }

    /**
     * Mètode que aplica l'idioma guardat a la persistència bàsica.
     */
    private void AplicaIdioma(){
        String idioma = new Opcions().getIdioma(this);
        Locale locale;
        if (idioma.equals(getString(R.string.catala))) {
            /*  Idioma Català */
            locale = new Locale("ca");
        }else {
             /*  Idioma Castellà */
            locale = new Locale("es");
        }
        Locale.setDefault(locale);
        Configuration config = new Configuration();
        config.locale = locale;
        getBaseContext().getResources().updateConfiguration(config, getBaseContext().getResources().getDisplayMetrics());
        super.onConfigurationChanged(config);
        setContentView(R.layout.activity_calendari);
        getActionBar().setTitle(getString(R.string.titolCalendari));
    }

    /**
     * Mètode que recupera el mes i l'any actual.
     */
    private void RecuperaMesAny(){
        /* Recuperar l'any i el mes actuals */
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy");
        any = Integer.valueOf(sdf.format(Calendar.getInstance().getTime()));
        sdf = new SimpleDateFormat("M");
        mes = Integer.valueOf(sdf.format(Calendar.getInstance().getTime()));
    }

    /**
     * Mètode per configurar el calendari. Mostra els botons corresponents
     * als dies que té el mes i els assigna el número de dia. Oculta els
     * que no són necessaris.
     */
    private void ActualitzaCalendari(){
        /* Calcular quin es el primer dia del mes i quants dies té */
        GregorianCalendar calendar;
        calendar = new GregorianCalendar(any, mes - 1, 1);
        primerDia = calendar.get(Calendar.DAY_OF_WEEK);
        calendar.add(Calendar.MONTH, 1);
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        int numDiesMes = calendar.get(Calendar.DAY_OF_MONTH);

        int limitInferior = new Opcions().getLimit(this, "Inferior");
        int limitSuperior = new Opcions().getLimit(this, "Superior");

        /* Mostrar el mes i l'any */
        String text1 = "";
        switch(mes){
            case 1: text1 = getString(R.string.gener); break;
            case 2: text1 = getString(R.string.febrer); break;
            case 3: text1 = getString(R.string.marz); break;
            case 4: text1 = getString(R.string.abril); break;
            case 5: text1 = getString(R.string.maig); break;
            case 6: text1 = getString(R.string.juny); break;
            case 7: text1 = getString(R.string.juliol); break;
            case 8: text1 = getString(R.string.agost); break;
            case 9: text1 = getString(R.string.setembre); break;
            case 10: text1 = getString(R.string.octubre); break;
            case 11: text1 = getString(R.string.novembre); break;
            case 12: text1 = getString(R.string.desembre); break;
        }
        TextView textData = (TextView)findViewById(R.id.textViewMesAny);
        textData.setText(text1 + " " + any);

        /* Omplir la matriu de botons amb els dies corresponents */
        /* Matriu amb els IDs del botons */
        int[] botons = {
            R.id.button0, R.id.button1, R.id.button2, R.id.button3, R.id.button4, R.id.button5,
            R.id.button6, R.id.button7, R.id.button8, R.id.button9, R.id.button10, R.id.button11,
            R.id.button12, R.id.button13, R.id.button14, R.id.button15, R.id.button16, R.id.button17,
            R.id.button18, R.id.button19, R.id.button20, R.id.button21, R.id.button22, R.id.button23,
            R.id.button24, R.id.button25, R.id.button26, R.id.button27, R.id.button28, R.id.button29,
            R.id.button30, R.id.button31, R.id.button32, R.id.button33, R.id.button34, R.id.button35,
            R.id.button36, R.id.button37, R.id.button38, R.id.button39, R.id.button40, R.id.button41 };

        /* Primer botó visible al calendari, correspon al dia 1 */
        int primerBotoVisible;
        if (primerDia == 1) {
            primerBotoVisible = 6;
        }else{
            primerBotoVisible = primerDia - 2;
        }
        /* Ultim botó visible al calendari, correspon a l'últim dia del mes */
        int ultimBotoVisible = primerBotoVisible + numDiesMes -1;

        OperacionsDiari BD = new OperacionsDiari(this);
        boolean error = false;
        /* Si s'han de colorejar els dies obrir base de dades */
        if (mostrarColor){
            try{
                BD.open();
            }catch (SQLException e){
                error = true;
                showAlert(getString(R.string.errorObrirSQLite));
            }
        }
        if (!error) {
            /* Establir les propietats de cada botó */
            for (int i = 0; i < 42; i++) {
                final Button boto = (Button) findViewById(botons[i]);
                if (i >= primerBotoVisible & i <= ultimBotoVisible) {
                    boto.setVisibility(View.VISIBLE);
                    boto.setText(String.valueOf(i - primerBotoVisible + 1));
                    if (mostrarColor) {
                        /* Mostrar color segons el valor mig del dia*/
                        String data = String.format("%04d", any) + "-" + String.format("%02d", mes) +
                                "-" + String.format("%02d", Integer.valueOf(boto.getText().toString()));
                        int valorMig = BD.ValorMig(data);
                        if (valorMig == 0) {
                            /* Aplicar color neutre */
                            boto.getBackground().setColorFilter(Color.rgb(230, 230, 230), PorterDuff.Mode.MULTIPLY);
                        } else {
                            if (valorMig < limitInferior) {
                                /* Aplicar filtre blau */
                                boto.getBackground().setColorFilter(Color.rgb(0, 0, 255), PorterDuff.Mode.MULTIPLY);
                            } else if (valorMig > limitSuperior) {
                                /* Aplicar filtre vermell */
                                boto.getBackground().setColorFilter(Color.rgb(255, 0, 0), PorterDuff.Mode.MULTIPLY);
                            } else {
                                /* Aplicar filtre verd */
                                boto.getBackground().setColorFilter(Color.rgb(0, 255, 0), PorterDuff.Mode.MULTIPLY);
                            }
                        }
                    } else {
                        /* Color per defecte */
                        boto.getBackground().setColorFilter(Color.rgb(230, 230, 230), PorterDuff.Mode.MULTIPLY);
                    }
                } else {
                    boto.setVisibility(View.INVISIBLE);
                    boto.setText("");
                }
            }
            if (mostrarColor) {
                BD.close();
            }
        }
    }

    /**
     * Mètode assignat al botó per anar al mes anterior.
     * @param v Button
     */
    public void btnEnrera (View v){
        /* Retrocedir un mes */
        mes--;
        if (mes == 0){
            mes = 12;
            any--;
        }
        ActualitzaCalendari();
    }

    /**
     * Mètode assignat al botó per anar al mes següent.
     * @param v Button
     */
    public void btnEndavant (View v){
        /* Avançar un mes */
        mes++;
        if (mes == 13){
            mes = 1;
            any++;
        }
        ActualitzaCalendari();
    }

    /**
     * Mètode assignat als botons del calendari. Llança l'activitat EntradaValos
     * i li passa la data seleccioanada dins el Intent (amb putExtra).
     * @param v Button
     */
    public void btnDia (View v){
        /* Recuperar el text del botó per saber el dia seleccionat */
        Button boto = (Button)findViewById(v.getId());
        int diaSeleccionat = Integer.valueOf(boto.getText().toString());
        /* Llança l'activitat EntradaDeValors */
        Intent intent = new Intent(Calendari.this, EntradaValors.class);
        intent.putExtra("dia", diaSeleccionat);
        intent.putExtra("mes", mes);
        intent.putExtra("any", any);
        intent.putExtra("primerDia", primerDia);
        startActivityForResult(intent, 0);
    }

    /**
     * Mètode que executa les aciones necessàries al tornar de les activitats llançades.
     * En cas de que l'activitat hagi modificat algun valor es mostra un tost per informar-ho.
     * En cas de canvi d'idioma es renicia l'activity per carregar els strings.xml corresponents.
     * @param requestCode int
     * @param resultCode int
     * @param data Intent
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data){
        if (resultCode == RESULT_OK){
            if (data.getExtras().getInt("IDIOMA_CANVIAT", 0) == 1){
                AplicaIdioma();
                RecuperaMesAny();
                ActualitzaCalendari();
                /* Reiniciar l'activity per carregar el nou idioma al layout i al menú */
                Intent i = getBaseContext().getPackageManager().getLaunchIntentForPackage( getBaseContext().getPackageName() );
                i.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                startActivity(i);
            }
            if (data.getExtras().getInt("DADES_ACTUALITZADES", 0) == 1){
                Toast.makeText(getApplicationContext(),getString(R.string.dadesActualitzadesOK),Toast.LENGTH_SHORT).show();
                /* Si està activat el color redibuixar el calendari */
                ActualitzaCalendari();
            }
        }
    }

    /**
     * Mètode assignat al botó SORTIR.
     * @param v Button
     */
    public void btnSortir (View v){
        sortir();
    }

    /**
     * Mètode auxiliar quan es vol sortir de l'aplicació.
     * Pregunta si es vol sortir i actua en funció de la resposta.
     */
    private void sortir(){
        /* Preguntar si es vol sortir de l'aplicació */
        /* Preparar el listener del dialog */
        DialogInterface.OnClickListener dialogClickListener = new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int opcio) {
                if (opcio == DialogInterface.BUTTON_POSITIVE){
                /* Sortir de l'aplicació */
                    System.exit(0);
                }
            }
        };
        AlertDialog.Builder alert = new AlertDialog.Builder(this);
        alert.setMessage(getString(R.string.sortirApp));
        alert.setPositiveButton(getString(R.string.SI), dialogClickListener);
        alert.setNegativeButton(getString(R.string.NO), dialogClickListener);
        alert.show();
    }

    /**
     * Mètode assignat al botó MENU.
     * @param v Button
     */
    public void btnMenu (View v){
        /* Activar el menú d'opcions igual que si es prem la tecla menú */
        this.openOptionsMenu();
    }

    /**
     * Metode per mostrar una alerta.
     * @param text Cadena amb el text a mostrar.
     */
    private void showAlert(String text) {
        AlertDialog.Builder alert = new AlertDialog.Builder(this);
        alert.setMessage(text);
        alert.setPositiveButton(getString(R.string.ACCEPTAR), null);
        alert.show();
    }
}
