package edu.uoc.tfc.glucodiari;


import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.os.Bundle;
import android.widget.AdapterView;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.Spinner;

/**
 * Aquesta classe permet canviar els valors límit i l'idioma.
 * @author: Xavier Asensio - (TFC App. Android - UOC).
 * @version: 1.0
 */

public class Configuracio extends Activity implements AdapterView.OnItemSelectedListener{

    private int[] limitsOriginals = new int[2];
    private String idiomaOriginal;
    private String idiomaSeleccionat;
    /* Matriu amb els IDs dels EditText */
    private int[] editTexts = {R.id.editLimitSuperior, R.id.editLimitInferior};
    /* Matriu amb els IDs de les etiquetes */
    private int[] etiquetes = {R.string.LimitSuperior, R.string.LimitInferior};

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        setContentView(R.layout.activity_configuracio);

        /* Actualitza títol ActionBar per si ha canviat l'idioma */
        getActionBar().setTitle(getString(R.string.titolConfiguracio));

        /* Recuperar el valors i l'idioma guardats a la persistència bàsica */
        Opcions opcions = new Opcions();
        limitsOriginals[0] = opcions.getLimit(this,"Superior");
        limitsOriginals[1] = opcions.getLimit(this, "Inferior");
        idiomaOriginal =  opcions.getIdioma(this);

        /* Carregar els valors en els editText */
        EditText edit = (EditText) findViewById(R.id.editLimitSuperior);
        edit.setText(String.valueOf(limitsOriginals[0]));
        edit = (EditText) findViewById(R.id.editLimitInferior);
        edit.setText(String.valueOf(limitsOriginals[1]));

        /* Activar el listener en l'spinner */
        Spinner spinner = (Spinner) findViewById(R.id.spinnerIdioma);
        spinner.setOnItemSelectedListener(this);;

        /* Carregar en l'spainner l'idioma actual */
        if (idiomaOriginal.equals(getString(R.string.catala))) {
            spinner.setSelection(0);
        }else{
            spinner.setSelection(1);
        }
    }

    /**
     * Mètode per canviar la bandera quan es selecciona un element de l'spinner.
     * */
    @Override
    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
        /* Recuperar el nom de l'idioma selecciona */
        idiomaSeleccionat = parent.getItemAtPosition(position).toString();
        /* Mostrar la bandera corresponent a l'idioma */
        ImageView image = (ImageView) findViewById(R.id.imageIdioma);
        switch(idiomaSeleccionat){
            case "Català": image.setImageResource(R.drawable.icona_catala); break;
            case "Castellano": image.setImageResource(R.drawable.icona_castella);break;
        }
    }

    /**
     * Mètode activat quan la selecció desapareix de l'espinner
     * No cal fer res.
     */
    @Override
    public void onNothingSelected(AdapterView<?> parent) {
     }

    /**
     * Mètode assignat a la tecla BACK de sota la pantalla.
     * Comprova si s'han modificat valors i preguna si es vol tornar
     * sense guardar-los.
     */
    @Override
    public void onBackPressed(){
        /* Si els valor s'han modificat preguntar si es volen cancel·lar les modificacions */
        if (valorsModificats()) {
             /* Preparar el listener del dialog */
            DialogInterface.OnClickListener dialogClickListener = new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialog, int opcio) {
                    if (opcio == DialogInterface.BUTTON_POSITIVE){
                    /* Tornar sense guardar modificacions */
                        Intent i = getIntent();
                        i.putExtra("DADES_ACTUALITZADES", 0);
                        setResult(RESULT_CANCELED, i);
                        finish();
                    }
                }
            };
            AlertDialog.Builder alert = new AlertDialog.Builder(this);
            alert.setMessage(getString(R.string.cancelarModificacions));
            alert.setPositiveButton(getString(R.string.SI), dialogClickListener);
            alert.setNegativeButton(getString(R.string.NO), dialogClickListener);
            alert.show();
        }else{
            /* Si els valor no s'han modificat es pot tornar a l'activitat anterior */
            Intent i = getIntent();
            i.putExtra("DADES_ACTUALITZADES", 0);
            setResult(RESULT_CANCELED, i);
            finish();
        }
    }

    /**
     * Mètode assignat al botó ACCEPTAR.
     * Comprova que els valors compleixen els requisits per ser acceptats.
     * i els guarda i torna a l'activity anterior.
     * @param v Button
     */
    public void btnAcceptar (View v){
        boolean error = false;
        /* Comprovar que els 2 valors estan entre 0 i 600 */
        for(int i = 0; i < 2; i++){
            EditText edit = (EditText)findViewById(editTexts[i]);
            /* Si no hi ha cap valor substituir per 0 */
            if (edit.getText().toString().length() == 0) edit.setText("0");
            /* Comprovar que el valor és dins de límits */
            if (0 > Integer.valueOf(edit.getText().toString()) | 600 < Integer.valueOf(edit.getText().toString())) {
                showAlert(getString(etiquetes[i]) + " " + getString(R.string.errorEnValor));
                error = true;
            }
        }
        /* Comprovar que el límit inferior és més petit que el superior */
        if (!error){
            EditText edit = (EditText)findViewById(editTexts[0]);
            int superior = Integer.valueOf(edit.getText().toString());
            edit = (EditText)findViewById(editTexts[1]);
            int inferior = Integer.valueOf(edit.getText().toString());
            if (inferior >= superior){
                showAlert(getString(R.string.errorInferiorSuperior));
                error = true;
            }
        }
        /* Si NO s'ha modificat els valors no cal guardar i es torna a l'activitat anterior */
        if (!valorsModificats()){
            Intent i = getIntent();
            i.putExtra("DADES_ACTUALITZADES", 0);
            setResult(RESULT_OK, i);
            finish();
        }else{
            /* Els valors s'han modificat.
             * Si no tenen error es guarden i es torna a l'activitat anterior */
            if (!error){
                guardarValors();
                Intent i = getIntent();
                i.putExtra("DADES_ACTUALITZADES", 1);
                if (!idiomaSeleccionat.equals(idiomaOriginal)){
                    i.putExtra("IDIOMA_CANVIAT", 1);
                }
                setResult(RESULT_OK, i);
                finish();
            }
        }
    }

    /**
     * Mètode auxiliar que comprova si els valors s'han modificat.
     * @return true si s'han modificat. false altrament.
     */
    private boolean valorsModificats(){
        boolean modificat = false;
        for (int i = 0; i < 2; i++) {
            EditText edit = (EditText) findViewById(editTexts[i]);
            if (limitsOriginals[i] != Integer.valueOf(edit.getText().toString())){
                /* Valor modificat */
                modificat = true;
            }
        }
        if (!idiomaSeleccionat.equals(idiomaOriginal)){
            modificat = true;
        }
        return modificat;
    }

    /**
     * Mètode auxiliar per guardar els valors a la persitència bàsica.
     * Fa servir per guardar-los la classe Opcions.
     */
    private void guardarValors(){
        Opcions opcions = new Opcions();
        EditText edit = (EditText) findViewById(editTexts[0]);
        opcions.setLimit(this, "Superior", Integer.valueOf(edit.getText().toString()));
        edit = (EditText) findViewById(editTexts[1]);
        opcions.setLimit(this, "Inferior", Integer.valueOf(edit.getText().toString()));
        opcions.setIdioma(this, idiomaSeleccionat);
    }

    /**
     * Metode per mostrar una alerta.
     * @param text Cadena amb el text a mostrar.
     */
    private void showAlert(String text) {
        AlertDialog.Builder alert = new AlertDialog.Builder(this);
        alert.setMessage(text);
        alert.setPositiveButton(getString(R.string.ACCEPTAR), null);
        alert.show();
    }
}

