package edu.uoc.tfc.glucodiari;


import android.app.Activity;
import android.app.AlertDialog;
import android.content.pm.ActivityInfo;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.HttpConnectionParams;
import org.json.JSONObject;


import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.os.StrictMode;
import android.view.View;
import android.widget.Button;

/**
 * Aquesta classe permet mostrar el codi d'usuari i enviar les dades al servidor.
 * @author: Xavier Asensio - (TFC App. Android - UOC).
 * @version: 1.0
 */


public class EinesDeDades extends Activity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        setContentView(R.layout.activity_einesdedades);

        /* Permís per accedir la xarxa a la classe HTTPClient */
        StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder().permitAll().build();
        StrictMode.setThreadPolicy(policy);

        /* Actualiza títol ActionBar per si ha canviat l'idioma */
        getActionBar().setTitle(getString(R.string.titolEinesDades));

        /* Mostrar el codi d'usuari al botó */
        Button boto = (Button) findViewById(R.id.buttonCodiUsuari);
        boto.setText(new Opcions().getCodiUsuari(this));

    }

    /**
     * Mètode associat al botó TORNAR AL CALENDARI.
     * @param v Button
     */
    public void btnTornarCalendari (View v){
        /* Tancar l'activitat per tornar al calendari */
        finish();
    }

    /**
     * Mètode associat al botó ENVIAR DADES.
     * Crea una cadena JSON i l'envia a insertvalues.php guardat al servidor.
     * @param v Button
     */
    public void btnEnviarDades (View v) {
        /* Comprovar si hi ha connexió a la xarxa */
        if (ConnexioXarxa()) {
            String usuari = new Opcions().getCodiUsuari(this);
            OperacionsDiari BD = new OperacionsDiari(this);
            try {
                BD.open();
            /* Recuperar el diari en JSON */
                String jsonString = BD.diariJSON(usuari).toString();
                BD.close();
            /* Envia el diari */
                if (ConnexioPHP("http://www.neko77.com/TFC_GlucoDiari/insertvalue.php", jsonString)) {
                    showAlert(getString(R.string.OKEnviarDadesServidor));
                } else {
                    showAlert(getString(R.string.ErrorEnviarDadesServidor));
                }
            } catch (Exception e) {
                showAlert(getString(R.string.errorObrirSQLite));
                BD.close();
                return;
            }
        }else{
            showAlert(getString(R.string.NoConnexioXarxa));
        }
    }

    /**
     * Mètode per crear la connexió HTTP i enviar la cadena JSON.
     */
    private boolean ConnexioPHP (String url, String cadenaJSON ) {
        boolean resultatPHP;
        HttpClient client = new DefaultHttpClient();
        HttpConnectionParams.setConnectionTimeout(client.getParams(), 100000);
        JSONObject jsonResponse = null;
        HttpPost post = new HttpPost(url);
        try {
            StringEntity se = new StringEntity("GlucoDiariJSON=" + cadenaJSON);
            post.addHeader("content-type", "application/x-www-form-urlencoded");
            post.setEntity(se);

            HttpResponse response;
            response = client.execute(post);
            String resFromServer = org.apache.http.util.EntityUtils.toString(response.getEntity());

            if(resFromServer.equals("OK")) {
                resultatPHP = true;
            }else{
                resultatPHP = false;
            };

        } catch (Exception e) {
            return false;
        }
        return resultatPHP;
    }

    /**
     * Mètode que comprova si hi ha connexió a la xarxa
     * @return true si hi ha connexió. false altrament.
     */
    public boolean ConnexioXarxa(){
        ConnectivityManager connMgr = (ConnectivityManager) getSystemService(this.CONNECTIVITY_SERVICE);
        NetworkInfo networkInfo = connMgr.getActiveNetworkInfo();
        if (networkInfo != null && networkInfo.isConnected())
            return true;
        else
            return false;
    }

    /**
     * Metode per mostrar una alerta.
     * @param text Cadena amb el text a mostrar.
     */
    private void showAlert(String text){
        AlertDialog.Builder alert = new AlertDialog.Builder(this);
        alert.setMessage(text);
        alert.setPositiveButton(getString(R.string.ACCEPTAR), null);
        alert.show();
    }
}
