package edu.uoc.tfc.glucodiari;


import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.os.Bundle;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;

import java.sql.SQLException;

/**
 * Aquesta classe permet mostrar els valors d'un dia donat i modificar-los.
 * @author: Xavier Asensio - (TFC App. Android - UOC).
 * @version: 1.0
 */

public class EntradaValors extends Activity {

    private int dia;
    private int mes;
    private int any;
    private int[] valorsOriginals = new int[4];
    private OperacionsDiari BD = new OperacionsDiari(this);
    /* Matriu amb els IDs dels EditText */
    private int[] editTexts = {R.id.editEsmorzar, R.id.editDinar, R.id.editSopar, R.id.editNit};
    /* Matriu amb els IDs de les etiquetes */
    private int[] etiquetes = {R.string.esmorzar, R.string.dinar, R.string.sopar, R.string.nit};

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        setContentView(R.layout.activity_entradavalors);

        /* Actualitza títol ActionBar per si ha canviat l'idioma */
        getActionBar().setTitle(getString(R.string.titolEntradaValors));

        /* Recuperar els valors guardats al Bundle */
        dia = getIntent().getExtras().getInt("dia");
        mes = getIntent().getExtras().getInt("mes");
        any = getIntent().getExtras().getInt("any");
        int primerDia = getIntent().getExtras().getInt("primerDia");

        /* Mostrar el nom del dia de la setmana */
        int nomDia = (primerDia + dia - 2 ) % 7;
        TextView textData = (TextView)findViewById(R.id.diaSetmana);
        switch(nomDia){
            case 0: textData.setText(getString(R.string.diumenge)); break;
            case 1: textData.setText(getString(R.string.dilluns)); break;
            case 2: textData.setText(getString(R.string.dimarts)); break;
            case 3: textData.setText(getString(R.string.dimecres)); break;
            case 4: textData.setText(getString(R.string.dijous)); break;
            case 5: textData.setText(getString(R.string.divendres)); break;
            case 6: textData.setText(getString(R.string.dissabte)); break;
        }

        /* Mostrar la data sencera */
        String text1 = "";
        switch(mes){
            case 1: text1 = getString(R.string.gener); break;
            case 2: text1 = getString(R.string.febrer); break;
            case 3: text1 = getString(R.string.marz); break;
            case 4: text1 = getString(R.string.abril); break;
            case 5: text1 = getString(R.string.maig); break;
            case 6: text1 = getString(R.string.juny); break;
            case 7: text1 = getString(R.string.juliol); break;
            case 8: text1 = getString(R.string.agost); break;
            case 9: text1 = getString(R.string.setembre); break;
            case 10: text1 = getString(R.string.octubre); break;
            case 11: text1 = getString(R.string.novembre); break;
            case 12: text1 = getString(R.string.desembre); break;
        }
        textData = (TextView)findViewById(R.id.diaMesAnyEntradaValors);
        textData.setText(dia + " " + text1 + " " + any);

        /* Obrir la base de dades */
        try{
            BD.open();
        }catch (SQLException e){
            showAlert(getString(R.string.errorObrirSQLite));
            return;
        }
        /* Recuperar els valor corresponents al dia seleccionat */
        int[] valors;
        try{
            String data = String.format("%04d", any) + "-" + String.format("%02d", mes) + "-" + String.format("%02d", dia);
            valors = BD.RecuperarRegistre(data);
            BD.close();
        }catch (SQLException e){
            showAlert(getString(R.string.errorRecuperarRegistre));
            return;
        }
        /* Assignar els valor als editText corresponents i a l'array valorsOriginals*/
        if(valors != null) {
            for (int i = 0; i < 4; i++) {
                EditText edit = (EditText) findViewById(editTexts[i]);
                valorsOriginals[i] = valors[i];
                edit.setText(String.valueOf(valorsOriginals[i]));
            }
        }
    }

    /**
     * Mètode assignat a la tecla BACK de sota la pantalla.
     * Comprova si s'han modificat valors i preguna si es vol tornar
     * sense guardar-los.
     */
    @Override
    public void onBackPressed(){
        /* Si els valor s'han modificat preguntar si es volen cancel·lar les modificacions */
        if (valorsModificats()) {
             /* Preparar el listener del dialog */
            DialogInterface.OnClickListener dialogClickListener = new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialog, int opcio) {
                    if (opcio == DialogInterface.BUTTON_POSITIVE){
                    /* Tornar sense guardar modificacions */
                        Intent i = getIntent();
                        i.putExtra("DADES_ACTUALITZADES", 0);
                        setResult(RESULT_CANCELED, i);
                        finish();
                    }
                }
            };
            AlertDialog.Builder alert = new AlertDialog.Builder(this);
            alert.setMessage(getString(R.string.cancelarModificacions));
            alert.setPositiveButton(getString(R.string.SI), dialogClickListener);
            alert.setNegativeButton(getString(R.string.NO), dialogClickListener);
            alert.show();
        }else{
            /* Si els valor no s'han modificat es pot tornar a l'activitat anterior */
            Intent i = getIntent();
            i.putExtra("DADES_ACTUALITZADES", 0);
            setResult(RESULT_CANCELED, i);
            finish();
        }
    }

    /**
     * Mètode assignat al botó ACCEPTAR.
     * Comprova que els valors compleixen els requisits per ser acceptats.
     * i els guarda i torna a l'activity anterior.
     * @param v Button
     */
    public void btnAcceptar (View v){
        boolean error = false;
        /* Comprovar que els 4 valors estan entre 0 i 600 */
        /* El rang de la majoria de glucòmetres va de 10-20 a 500-600 mg/dl */
        for(int i = 0; i < 4; i++){
            EditText edit = (EditText)findViewById(editTexts[i]);
            /* Si no hi ha cap valor substituir per 0 */
            if (edit.getText().toString().length() == 0) edit.setText("0");
            /* Comprovar que el valor és dins de límits */
            if (0 > Integer.valueOf(edit.getText().toString()) | 600 < Integer.valueOf(edit.getText().toString())) {
                showAlert(getString(etiquetes[i]) + " " + getString(R.string.errorEnValor));
                error = true;
            }
        }
        /* Si NO s'ha modificat els valors no cal guardar i es torna a l'activitat anterior */
        if (!valorsModificats()){
            Intent i = getIntent();
            i.putExtra("DADES_ACTUALITZADES", 0);
            setResult(RESULT_OK, i);
            finish();
        }else{
            /* Els valors s'han modificat.
             * Si no tenen error es guarden i es torna a l'activitat anterior */
            if (!error){
                if (guardarValors()){
                    /* Valors guardats OK */
                    Intent i = getIntent();
                    i.putExtra("DADES_ACTUALITZADES", 1);
                    setResult(RESULT_OK, i);
                    finish();
                }else{
                    showAlert(getString(R.string.errorInserirRegistre));
                }
            }
        }
    }

    /**
     * Mètode auxiliar que comprova si els valors s'han modificat.
     * @return true si s'han modificat. false altrament.
     */
    private boolean valorsModificats(){
        boolean modificat = false;
        for (int i = 0; i < 4; i++) {
            EditText edit = (EditText) findViewById(editTexts[i]);
            if (valorsOriginals[i] != Integer.valueOf(edit.getText().toString())){
                /* Valor modificat */
                modificat = true;
            }
        }
        return modificat;
    }

    /**
     * Mètode auxiliar per guardar els valors en la BD.
     * @return true si guardats OK. false altrament.
     */
    private boolean guardarValors(){
        int[] valors = new int[4];
        for (int i = 0; i < 4; i++) {
            EditText edit = (EditText) findViewById(editTexts[i]);
            valors[i] = Integer.valueOf(edit.getText().toString());
        }
        /* Obrir la base de dades */
        try{
            BD.open();
        }catch (SQLException e){
            showAlert(getString(R.string.errorObrirSQLite));
            return false;
        }
        /* Guardar els valors */
        try {
            String data = String.format("%04d", any) + "-" + String.format("%02d", mes) + "-" + String.format("%02d", dia);
            if (!BD.InserirRegistre(data, valors)) {
                /* Error en la inserció de les dades */
                showAlert(getString(R.string.errorInserirRegistre));
                BD.close();
                return false;
            } else {
                /* Inserció de les dades OK */
                BD.close();
                return true;
            }
        } catch (SQLException e) {
            showAlert(getString(R.string.errorInserirRegistre));
            return false;
        }
    }

    /**
     * Metode per mostrar una alerta.
     * @param text Cadena amb el text a mostrar.
     */
    private void showAlert(String text){
        AlertDialog.Builder alert = new AlertDialog.Builder(this);
        alert.setMessage(text);
        alert.setPositiveButton(getString(R.string.ACCEPTAR), null);
        alert.show();
    }
}
