package edu.uoc.tfc.glucodiari;


import android.app.Activity;
import android.app.AlertDialog;
import android.content.pm.ActivityInfo;
import android.os.Bundle;
import android.view.View;
import android.widget.TextView;

import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 * Aquesta classe permet recuperar i mostrar els informes dels
 * últims 7 i 30 dies i mostar-los.
 * @author: Xavier Asensio - (TFC App. Android - UOC).
 * @version: 1.0
 */

public class Informes extends Activity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        setContentView(R.layout.activity_informes);

        /* Carregar les dades als editText dels últims 7 dies */
        OmplirDades(7, R.id.txtUltims7NombreRegistres, R.id.txtUltims7ValorMig, R.id.txtUltims7ValorsSobreLimit,
                R.id.txtUltims7ValorsSotaLimit, R.id.txtUltims7ValorsDinsLimits);

        /* Carregar les dades als editText dels últims 30 dies */
        OmplirDades(30, R.id.txtUltims30NombreRegistres, R.id.txtUltims30ValorMig,R.id.txtUltims30ValorsSobreLimit,
                R.id.txtUltims30ValorsSotaLimit, R.id.txtUltims30ValorsDinsLimits);
    }

    /**
     * Mètode auxiliar per omplir els textView amb els valors corresponents als informes.
     * @param nombreDies nombre de dies enrera desde la data actual.
     * @param idNombreRegistres id del textView nombre de registres.
     * @param idValorMig id del textView valor mig.
     * @param idValorsSobre id del textView valors sobre el límit.
     * @param idValorsSota id del textView valors sota el límit.
     * @param idValorsDins id del textView valors dintre de límits.
     */
    private void OmplirDades(int nombreDies, int idNombreRegistres, int idValorMig, int idValorsSobre,
                             int idValorsSota, int idValorsDins){
        OperacionsDiari BD = new OperacionsDiari(this);
        /* Obrir la base de dades */
        try{
            BD.open();
        }catch (SQLException e){
            showAlert(getString(R.string.errorObrirSQLite));
            return;
        }

        /* Recuperar els valors dels límits */
        int limitSuperior = new Opcions().getLimit(this, "Superior");
        int limitInferior = new Opcions().getLimit(this, "Inferior");

        /* Recuperar l'any i el mes actuals */
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy");
        int any = Integer.valueOf(sdf.format(Calendar.getInstance().getTime()));
        sdf = new SimpleDateFormat("M");
        int mes = Integer.valueOf(sdf.format(Calendar.getInstance().getTime()));
        sdf = new SimpleDateFormat("dd");
        int dia = Integer.valueOf(sdf.format(Calendar.getInstance().getTime()));
        /* Calcular la data n dies enrera */
        /* També es tindran en compte els valors d'avui */
        GregorianCalendar calendar;
        calendar = new GregorianCalendar(any, mes - 1, dia);
        calendar.add(Calendar.DAY_OF_MONTH, - nombreDies);
        int diaAnterior = calendar.get(Calendar.DAY_OF_MONTH);
        int mesAnterior = calendar.get(Calendar.MONTH) + 1;
        int anyAnterior = calendar.get(Calendar.YEAR);
        /* Recuperar els valor de la BD */
        int[] resultats;
        try {
            String dataFi = String.format("%04d", any) + "-" + String.format("%02d", mes) + "-" + String.format("%02d", dia);
            String dataInici = String.format("%04d", anyAnterior) + "-" + String.format("%02d", mesAnterior) + "-" + String.format("%02d", diaAnterior);
            resultats = BD.RegistresUltimsDies(dataInici, dataFi, limitSuperior, limitInferior);
            BD.close();
        }catch (SQLException e){
            showAlert(getString(R.string.errorRecuperarRegistre));
            BD.close();
            return;
        }
        /* Carregar els resultats en els textView */
        TextView text = (TextView) findViewById(idNombreRegistres);
        text.setText(String.valueOf(resultats[0]));
        text = (TextView) findViewById(idValorMig);
        if (resultats[0] != 0) {
            text.setText(String.valueOf(resultats[1] / resultats[0]));
        }else{
            text.setText("0");
        }
        text = (TextView) findViewById(idValorsSobre);
        text.setText(String.valueOf(resultats[2]));
        text = (TextView) findViewById(idValorsSota);
        text.setText(String.valueOf(resultats[3]));
        text = (TextView) findViewById(idValorsDins);
        if (resultats[0] != 0) {
            int tantPerCent = ((resultats[0] - resultats[2] - resultats[3]) * 100) / resultats[0] ;
            text.setText(String.valueOf(tantPerCent)+ " %");
        }else{
            text.setText("0 %");
        }
    }

    /**
     * Mètode associat al botó TORNAR AL CALENDARI.
     * @param v Button
     */
    public void btnTornarCalendari (View v){
        /* Tancar l'activitat per tornar al calendari */
        finish();
    }

    /**
     * Metode per mostrar una alerta.
     * @param text Cadena amb el text a mostrar.
     */
    private void showAlert(String text){
        AlertDialog.Builder alert = new AlertDialog.Builder(this);
        alert.setMessage(text);
        alert.setPositiveButton(getString(R.string.ACCEPTAR), null);
        alert.show();
    }
}
