package edu.uoc.tfc.glucodiari;

import android.content.Context;
import android.content.SharedPreferences;
import java.util.Random;

/**
 * Aquesta classe permet recuperar i guardar els valor de configuració
 * fent servir la persistencia bàsica per guardar les dades.
 * @author: Xavier Asensio - (TFC App. Android - UOC).
 * @version: 1.0
 * @see <a href = "http://developer.android.com/guide/topics/data/data-storage.html#pref" /> Storage Options </a>
 */

public class Opcions{

    /**
     * Constructor
     */
    public void Opcions(){

    }

    /**
     * Mètode que retorna el límit demanat al paràmetre tipus.
     * @param context Context.
     * @param tipus Tipus de límit demanat (Inferior o Superior).
     * @return Valor del límit demanat.
     */
    public int getLimit(Context context, String tipus){
        /* Retorna el valor límit demanat a tipus */
        SharedPreferences preferences;
        preferences = context.getSharedPreferences("Limits", 0);
        int valor;
        if ("Inferior".equals(tipus)) {
            /* Recuperar límit inferior, valor per defecte 80 */
            valor = preferences.getInt(tipus, 80);
        }else{
            /* Recuperar límit superior, valor per defecte 150 */
            valor = preferences.getInt(tipus, 150);
        }
        return valor;
    }

    /**
     * Mètode que guarda el valor del límit indicat.
     * @param context Context.
     * @param tipus Tipus de límit que es vol guardar (Inferior o Superior).
     * @param valor Valor del límit que es vol guardar.
     */
    public void setLimit(Context context, String tipus, int valor){
        /* Guarda el valor límit indicat a tipus */
        SharedPreferences preferences;
        preferences = context.getSharedPreferences("Limits", 0);
        /* Guardar el nou valor del límit */
        preferences.edit().putInt(tipus, valor).apply();
    }

    /**
     * Mètode que retorna el codi d'usuari. Si el codi no existeix degut
     * a ser la primera vegada que es consulta, es genera un de nou aleatòriament.
     * @param context Context.
     * @return Valor del codi d'usuari. Es composa de sis caràcters hexadecinal.
     */
    public String getCodiUsuari(Context context){
        /* Retorna el codi d'usuari, si no existeix, el genera */
        SharedPreferences preferences;
        preferences = context.getSharedPreferences("Usuari", 0);
        String usuari;
        usuari = preferences.getString("Codi", "SENSE_CODI_USUARI");
        if ("SENSE_CODI_USUARI".equals(usuari)) {
            /* No té codi d'usuari, generar un de nou */
            usuari = "";
            /* Generar sis caracters hexadecimals aleatoris */
            Random r = new Random();
            int r1;
            for (int i = 0; i < 6; i++) {
                r1 = r.nextInt(15);
                switch (r1) {
                    case 10:
                        usuari = usuari + "A";
                        break;
                    case 11:
                        usuari = usuari + "B";
                        break;
                    case 12:
                        usuari = usuari + "C";
                        break;
                    case 13:
                        usuari = usuari + "D";
                        break;
                    case 14:
                        usuari = usuari + "E";
                        break;
                    case 15:
                        usuari = usuari + "F";
                        break;
                    default:
                        /* Valors de 0 a 9 */
                        usuari = usuari + String.valueOf(r1);
                        break;
                }
            }
            /* Guardar el codi d'usuari generat */
            preferences.edit().putString("Codi", usuari).apply();
        }
        return usuari;
    }

    /**
     * Mètode que retorna el nom de l'idioma.
     * @param context Context.
     * @return Nom de l'idioma.
     */
    public String getIdioma(Context context){
        /* Retorna l'idioma seleccionat, si no n'hi ha cap, retorna català */
        SharedPreferences preferences;
        preferences = context.getSharedPreferences("Usuari", 0);
        String idioma;
        idioma = preferences.getString("Idioma", context.getString(R.string.catala));
        return idioma;
    }

    /**
     * Mètode que guarda el nom de l'idioma indicat.
     * @param context Context.
     * @param idioma Nom de l'idioma que es vol guardar.
     */
    public void setIdioma(Context context, String idioma){
        /* Guarda l'idioma indicat */
        SharedPreferences preferences;
        preferences = context.getSharedPreferences("Usuari", 0);
        /* Guardar el nou valor de l'idioma */
        preferences.edit().putString("Idioma", idioma).apply();
    }

    /**
     * Mètode que retorna si es vol colorejar el calendari
     * @param context Context.
     * @return true si es vol colorejar. false altrament.
     */
    public boolean getColorejar(Context context){
        SharedPreferences preferences;
        preferences = context.getSharedPreferences("Usuari", 0);
        String colorejar;
        colorejar = preferences.getString("Colorejar", "NO");
        if (colorejar.equals(("SI"))){
            return true;
        }else{
            return false;
        }
    }

    /**
     * Mètode que guarda si es vol colorejar el calendari.
     * @param context Context.
     * @param valor true si es volor colorejar. false altrament.
     */
    public void setColorejar(Context context, boolean valor) {
        SharedPreferences preferences;
        preferences = context.getSharedPreferences("Usuari", 0);
        if (valor) {
            preferences.edit().putString("Colorejar", "SI").apply();
        } else {
            preferences.edit().putString("Colorejar", "NO").apply();
        }
    }
}
